/*
 * rockantenne.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

/*
 * CLASSIC mode is a http-get directly from Radio Rockantenne server
 * the standard mode uses an alternate server which provides more titles
 *
#define CLASSIC
 */

#include "rockantenne.h"
#include <vdr/plugin.h>
#include <vdr/skins.h>
#include <vdr/interface.h>
#include <vdr/osd.h>
#include <vdr/channels.h>
#include <vdr/device.h>

#define CACHETIME 10
#define TIMEOUT_SECS 10
/* very ugly, but very simple X-) */
#include "get_song_data.c"

#define ROCKANTENNE_FREQ 12149
#define ROCKANTENNE_VPID 0
#define ROCKANTENNE_APID 304

static const char *VERSION        = "0.2.2";
static const char *DESCRIPTION    = "Show song(s) played on Rockantenne";
static const char *MAINMENUENTRY  = "Rockantenne";

class cRockantenneDisplay : public cOsdObject {
private:
  cOsd *osd;
  const cFont* pFont;
  int lineHeight;
  long *psongtime;
  char *song;
public:
  cRockantenneDisplay(long *pst, char *s);
  ~cRockantenneDisplay();
  virtual void Show(void);
  virtual eOSState ProcessKey(eKeys Key);
};

//cRockantenneDisplay::cRockantenneDisplay(void) : cOsdObject(true) {
cRockantenneDisplay::cRockantenneDisplay(long *pst, char *s) {
  osd = NULL;
  pFont = NULL;
  psongtime = pst;
  song = s;
}

cRockantenneDisplay::~cRockantenneDisplay() {
  delete osd;
}

void cRockantenneDisplay::Show(void) {
  register char *p;
  register int i;
  register char c;
  register int lines;
  char *p2;
  int height, width;
  char http_buf[BUFLEN+25];
  struct song_data stsong;
  long ttime;

  if (!pFont) {
    pFont = cFont::GetFont(fontSml);
    pFont->SetCode(code_iso8859_1);
    lineHeight = pFont->Height();
  }
  i = lineHeight;
  const int screenWidth = Setup.OSDWidth;
  const int screenHeight = Setup.OSDHeight;
  width = (((int) (screenWidth / 13)) * 12 ) - 1 ;
  cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
  // I wonder if current channel is ROCKANTENNE
  // yes => use whole screen
  if ((channel->Frequency() == ROCKANTENNE_FREQ) &&
      (channel->Vpid() == ROCKANTENNE_VPID) &&
      (channel->Apid(0) == ROCKANTENNE_APID)) {
    lines = ((int)(screenHeight / 13)) * 12 / i;
    if (lines > 25) lines = 25;
    height = lines * i;
    osd = cOsdProvider::NewOsd(Setup.OSDLeft+((screenWidth - width) / 2),
                               Setup.OSDTop+((screenHeight-height) / 2));
  } else {
    lines = 4;
    height = lines * i;
    osd = cOsdProvider::NewOsd(Setup.OSDLeft+((screenWidth - width) / 2),
                               Setup.OSDTop+screenHeight-(lines*i)-10);
  }
  if (osd) {
    tArea Areas[] = { { 0, 0, width, (lines*i)-1, 2 } };
    if (osd->SetAreas(Areas, 1) != oeOk) {
      delete osd;
      return;
    }
    osd->DrawRectangle(0, 0, width, lines*lineHeight, clrGray50);
    --lines; // for rockantenne.de line

    // rockantenne.de or cache access
    ttime = get_time_long();
    if (ttime > (*psongtime+CACHETIME)) {
      i = do_http_get(http_buf, BUFLEN);
      if (i) {
        i = get_song_data(http_buf, i, &stsong);
        if (i!=0) {
          osd->DrawText(0,lineHeight*lines,"<error while parsing http data>", clrRed, clrTransparent, pFont);
        } else {
          p = song;
          i = str_copyn(p, "rockantenne.de: ", STRING_LEN/3);
          p += i;
          i = str_copyn(p, stsong.interpret, STRING_LEN/3);
          p += i;
          i = str_copyn(p, stsong.title, STRING_LEN/3);
          p[i] = '\0';
          p = song;
          // limit string
          p[64] = '\0';
          *psongtime = ttime;
          osd->DrawText(0,lineHeight*lines,p, clrYellow, clrTransparent, pFont);
        }
      } else {
        osd->DrawText(0,lineHeight*lines,"<error on http-get>", clrRed, clrTransparent, pFont);
      }
    } else {
      osd->DrawText(0,lineHeight*lines,song, clrYellow, clrTransparent, pFont);
    }

    // access to alternate server with more lines of data
    p = http_buf;
    for(i=0;i<BUFLEN+25;i++) p[i]='\n'; // "clear" buffer
    i = do_net_get(p, BUFLEN, "81.89.239.234", 4489, lines);
    if (i<=0) {
      // since v.0.2.2 net errors will be displayed on last osd line
      //delete osd;
      //Skins.Message(mtError, "<error on internet connection>");
      //Interface->Wait(Setup.OSDMessageTime);
      osd->Flush();
      return;
    }
    for(i=0;i<lines;i++) {
      p2 = p;
      c = ' ';
      while(*p!='\n') {
        if (*p=='\t') { *p = c; c = ':'; }
        ++p;
      }
      *p = '\0';
      osd->DrawText(0,lineHeight*i,p2, clrYellow, clrTransparent, pFont);
      ++p;
    }
    osd->Flush();
  }
}

eOSState cRockantenneDisplay::ProcessKey(eKeys Key) {
  eOSState state = cOsdObject::ProcessKey(Key);
  if (state == osUnknown) {
    switch (Key & ~k_Repeat) {
      case kOk: case kBack: return osEnd;
      default: return state;
    }
  }
  return state;
}

class cPluginRockantenne : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  struct song_data stsong;
  char http_buf[BUFLEN+1];
  char song[STRING_LEN+1];
  long songtime;
public:
  cPluginRockantenne(void);
  virtual ~cPluginRockantenne();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

cPluginRockantenne::cPluginRockantenne(void) {
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  songtime = 0;
}

cPluginRockantenne::~cPluginRockantenne() {
  // Clean up after yourself!
}

const char *cPluginRockantenne::CommandLineHelp(void) {
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginRockantenne::ProcessArgs(int argc, char *argv[]) {
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginRockantenne::Initialize(void) {
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginRockantenne::Start(void) {
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginRockantenne::Stop(void) {
  // Stop any background activities the plugin shall perform.
}

void cPluginRockantenne::Housekeeping(void) {
  // Perform any cleanup or other regular tasks.
}

void cPluginRockantenne::MainThreadHook(void) {
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!
}

cString cPluginRockantenne::Active(void) {
  // Return a message string if shutdown should be postponed
  return NULL;
}

cOsdObject *cPluginRockantenne::MainMenuAction(void) {

  // Perform the action when selected from the main VDR menu.
  return new cRockantenneDisplay(&songtime, song);
}

cMenuSetupPage *cPluginRockantenne::SetupMenu(void) {
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginRockantenne::SetupParse(const char *Name, const char *Value) {
  // Parse your own setup parameters and store their values.
  return false;
}

bool cPluginRockantenne::Service(const char *Id, void *Data) {
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginRockantenne::SVDRPHelpPages(void) {
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginRockantenne::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode) {
  // Process SVDRP commands this plugin implements
  return NULL;
}

VDRPLUGINCREATOR(cPluginRockantenne); // Don't touch this!
